package sim.app.heatbugs;
import sim.engine.*;
import sim.display.*;
import sim.portrayal.grid.*;
import java.awt.*;
import javax.swing.*;

public class HeatBugsWithUI extends GUIState
    {
    public Display2D display;
    public JFrame displayFrame;

    FastValueGrid2DPortrayal heatPortrayal = new FastValueGrid2DPortrayal("Heat");
    SparseGrid2DPortrayal bugPortrayal = new SparseGrid2DPortrayal();

    public static void main(String[] args)
        {
        HeatBugsWithUI heatbugs = new HeatBugsWithUI();
        Console c = new Console(heatbugs);
        c.setVisible(true);
        }
    
    public HeatBugsWithUI() { super(new HeatBugs(System.currentTimeMillis())); }
    
    public HeatBugsWithUI(SimState state) { super(state); }
    
    public String getName() { return "Heat Bugs"; }
    
    public Object getSimulationInspectedObject() { return state; }  // non-volatile

    public String getInfo()
        {
        
        HeatBugs hb = (HeatBugs) state;
        
        return 
        "<H2>HeatBugs</H2>by Sean Luke.<p>Part of the little simulator we're building.  Let's test the HTML output.  For the heck, the current values are:" +
        "<p><table><tr><td bgcolor=\"#3366CC\">Name</td><td bgcolor=\"#3366CC\">Value</td></tr>"+
        "<tr><td>Grid Height</td><td>" + hb.gridHeight  + "</td></tr>" +
        "<tr><td>Grid Width</td><td>" + hb.gridWidth  + "</td></tr>" +
        "<tr><td>Number of bugs</td><td>" + hb.bugCount  + "</td></tr>" +
        "</table>"; 
        }
    
    public void start()
        {
        super.start();
        // set up our portrayals
        setupPortrayals();
        }
    
    public void load(SimState state)
        {
        super.load(state);
        // we now have new grids.  Set up the portrayals to reflect that
        setupPortrayals();
        }
        
    // This is called by start() and by load() because they both had this code
    // so I didn't have to type it twice :-)
    public void setupPortrayals()
        {
        // tell the portrayals what to portray and how to portray them
        heatPortrayal.setField(((HeatBugs)state).valgrid);
        heatPortrayal.setLevels(0,HeatBugs.MAX_HEAT,Color.black,Color.red);
        bugPortrayal.setField(((HeatBugs)state).buggrid);
        bugPortrayal.setPortrayalForAll( new sim.portrayal.simple.OvalPortrayal2D(Color.white) );   // all the heatbugs will be white ovals
            
        // reschedule the displayer
        display.reset();
                
        // redraw the display
        display.repaint();
        }
    
    public void init(Controller c)
        {
        super.init(c);
        
        // Make the Display2D.  We'll have it display stuff later.
        display = new Display2D(400,400,this,1); // at 400x400, we've got 4x4 per array position
        displayFrame = display.createFrame();
        c.registerFrame(displayFrame);   // register the frame so it appears in the "Display" list
        displayFrame.setVisible(true);

        // attach the portrayals
        display.attach(heatPortrayal,"Heat");
        display.attach(bugPortrayal,"Bugs");

        // specify the backdrop color  -- what gets painted behind the displays
        display.setBackdrop(Color.black);
        }
        
    public void quit()
        {
        super.quit();
        
        if (displayFrame!=null) displayFrame.dispose();
        displayFrame = null;  // let gc
        display = null;       // let gc
        }
    }
    
    
    
    
    
